/*************************************************************************************
*  Copyright (c) 1997-1998 by Joseph Parrello. All Rights Reserved.                  *
*  Site   : http://space.tin.it/computer/giparrel                                    *
*  E-Mail : joparrel@tin.it, giusparr@tin.it, giparre@tin.it                         *
*                                                                                    *
*  Revision:   1.2a  (April 3, 1998).                                                *
*                                                                                    *
*  Definition of JPFlatButtonGadget class.                                           *
*                                                                                    *
*  Read JPFLTBTG.TXT for use and license.                                            *
*                                                                                    *
*  Only for OWL 5.02 and 5.01 users.                                                 *
*  OWL and ObjectWindows are trademarks of Borland International.                    *
*                                                                                    *
*  THIS CLASS AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF                *
*  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO               *
*  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A                    *
*  PARTICULAR PURPOSE.                                                               *
*************************************************************************************/
#include <owl/pch.h>

#if !defined(OWL_UIHELPER_H)
#include <owl/uihelper.h>
#endif

#include "jpfltbtg.h"


//{{JPFlatButtonGadget Implementation}}


JPFlatButtonGadget::JPFlatButtonGadget(TResId glyphResIdOrIndex, int id, TType type, bool enabled, TState state, bool sharedGlyph)
:
    TButtonGadget(glyphResIdOrIndex, id, type, enabled, state, sharedGlyph)
{
  // INSERT>> Your constructor code here.
  	TBorders borders;
  	borders.Top = borders.Left = 1;
  	borders.Right = borders.Bottom = 1;
  	SetBorders(borders);
   isEnter = false;
   if (State==Down)
   	SetFlatState(fsDown);
   else SetFlatState(fsNormal);
}


JPFlatButtonGadget::~JPFlatButtonGadget()
{
  // INSERT>> Your destructor code here.

}


void JPFlatButtonGadget::PaintBorder(TDC& dc)
{
    uint fstate = GetFlatState();
    uint nstate = GetButtonState();
    if ((fstate != fsUp) && (GetButtonType() != Command) && (!Pressed))
    	{
      if ((nstate == Down) && (fstate != fsDown))
      	{
         SetFlatState(fsDown, false);
         InvalidateRect(TRect(0, 0, Bounds.Width(), Bounds.Height()), true);
         }
      else if ((nstate == Up) && (fstate != fsNormal))
         {
         SetFlatState(fsNormal, false);
   		InvalidateRect(TRect(0, 0, Bounds.Width(), Bounds.Height()), true);
         }
      }
   if (GetFlatState() == fsUp)
      {
      TRect rect(0,0,Bounds.Width(),Bounds.Height());
      #if defined(BI_PLAT_WIN16)
      TUIBorder border(rect, TUIBorder::TEdge(TUIBorder::WndRaised), TUIBorder::Rect | TUIBorder::Adjust);
      #elif defined(BI_PLAT_WIN32)
      TUIBorder border(rect, TUIBorder::TEdge(TUIBorder::RaisedInner), TUIBorder::Rect | TUIBorder::Adjust);
      #endif
      border.Paint(dc);
      }
   else if (GetFlatState() == fsDown)
      {
      BorderStyle = Recessed;
      TButtonGadget::PaintBorder(dc);
      }
}

void
JPFlatButtonGadget::SetFlatState(FlatState state, bool update)
{
switch (state)
	{
   case fsNormal:
      CurrentFlatState = fsNormal;
      break;

   case fsUp:
      if (GetEnabled())
      CurrentFlatState = fsUp;
      else return;
      break;

   case fsDown:
      if (GetEnabled())
      CurrentFlatState = fsDown;
      else return;
      break;
   }
if (update)
	{
   InvalidateRect(TRect(0, 0, Bounds.Width(), Bounds.Height()), true);
	Update();
   }
}

JPFlatButtonGadget::FlatState
JPFlatButtonGadget::GetFlatState() const
{
  return  CurrentFlatState;
}


void JPFlatButtonGadget::MouseEnter(uint modKeys, TPoint& point)
{
  TButtonGadget::MouseEnter(modKeys, point);

  // INSERT>> Your code here.
  isEnter = true;
  if ((GetButtonState()==Up) && (GetEnabled()))
  		SetFlatState(fsUp);
}


void JPFlatButtonGadget::MouseLeave(uint modKeys, TPoint& point)
{
  TButtonGadget::MouseLeave(modKeys, point);

  // INSERT>> Your code here.
  isEnter = false;
   if ((State!=Down) && (GetEnabled()))
      SetFlatState(fsNormal);
}


void JPFlatButtonGadget::BeginPressed(TPoint& p)
{
  SetFlatState(fsDown , false);
  TButtonGadget::BeginPressed(p);
}


void JPFlatButtonGadget::CancelPressed(TPoint& p)
{
  bool oldpressed = Pressed;
  if ((State==Up) && (oldpressed))
  		{
      bool hit = PtIn(p);
      if ((hit) && GetEnabled())
  				SetFlatState(fsUp);
      else
  			SetFlatState(fsNormal);
      }
  TButtonGadget::CancelPressed(p);
}


void
JPFlatButtonGadget::SetButtonState(TState newState)
{
  TButtonGadget::SetButtonState(newState);
   if (State==Down)
      SetFlatState(fsDown);
   else
      SetFlatState(fsNormal);
}

bool
JPFlatButtonGadget::IdleAction(long idleCount)
{
  if (idleCount == 0)
    {
    if (GetEnabled()==false)
    	{
    	if ((GetFlatState()!=fsNormal) && (GetButtonState()==Up) && (GetButtonType()==Command))
    	   SetFlatState(fsNormal);
      }
    else
    	{
      if (isEnter==true)
      	{
      	if ((GetFlatState()==fsNormal) && (GetButtonState()==Up))
      		SetFlatState(fsUp);
         else if ((GetFlatState()==fsUp) && (GetButtonState()==Down))
         	SetFlatState(fsDown);
         }
      }
    }
  return TButtonGadget::IdleAction(idleCount);
}

